"""
Backend CourseOverview file, here are all the methods from
openedx.core.djangoapps.content.course_overviews.
"""

from unittest.mock import Mock


def get_course_overview():
    """Backend to get course overview."""

    return Mock()
