# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-06-30 13:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eox_tagging', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tag',
            name='activation_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='tag',
            name='expiration_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
